#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)

import sys
import logging
import os
import subprocess
from datetime import datetime
from cdo import Cdo

# Local import
from utils.utils_files import *

"""
Class: Spatial consistency checks
---
- Uses CDO python bindings
- Loops over all timestamps within the dataset
- Checks they are within the time range defined as input
"""
class temporal_completeness_test:

    def __init__(self, logdir, path, tstart, tend):
        self.testid = 'test03-temporal-completeness'
        self.logdir = logdir 
        self.path = path        
        self.files = dir_or_file(path)
        self.tstart = tstart
        self.tend = tend
        self.tstart_dt = datetime.strptime(tstart, '%Y-%m-%d')
        self.tend_dt = datetime.strptime(tend, '%Y-%m-%d')

    def check_timestamps(self):

    	# For all files
    	cdo = Cdo()
    	res = True
    	for f in self.files:
    		logging.info('Checking {}'.format(f))    		
    		try:
    			info = cdo.showtimestamp(input=f)
    		except:
    			logging.error('Bad file format (cdo cannot read file)')
    			print('ERROR: Bad file format ({})'.format(os.path.basename(f)))
    			res = False
    			break
	    	
	    	for i in info:
	    		for t in i.split():
	    			date = datetime.strptime(t, '%Y-%m-%dT%H:%M:%S')	    			
	    			if not(date >= self.tstart_dt and date <= self.tend_dt):	    			
	    				logging.error('Date out of range {} [{} -> {}]'.format(t, self.tstart, self.tend))	    				
	    				res = False

    	# Final message
    	if res:
    		logging.info('FILES ARE TEMPORALLY CONSISTENT')
    		print('INFO: FILES ARE TEMPORALLY CONSISTENT')
    	else:
    		logging.error('TEMPORAL INCONSISTENCIES FOUND')
    		print('TEMPORAL INCONSISTENCIES FOUND')

    def main(self):
        # Initiate the appropiate logs
    	initiate_logging(self.logdir, self.path, self.testid)

    	self.check_timestamps()
                
if __name__ == "__main__":
    sys.exit()
